==> onegov.feriennet-0.1.0.tar.gz: onegov.feriennet-0.1.0/onegov/feriennet/forms/attendee.py <==
from datetime import date
from onegov.activity import Attendee, AttendeeCollection
from onegov.activity import Booking, BookingCollection
from onegov.feriennet import _
from onegov.form import Form
from wtforms.fields import RadioField, StringField
from wtforms.fields.html5 import DateField
from wtforms.validators import InputRequired


class AttendeeForm(Form):

    attendee = RadioField(
        label=_("Attendee"),
        validators=[InputRequired()],
        default='0xdeadbeef')

    name = StringField(
        label=_("Full Name"),
        validators=[InputRequired()],
        depends_on=('attendee', 'other'))

    birth_date = DateField(
        label=_("Birthdate"),
        validators=[InputRequired()],
        depends_on=('attendee', 'other'))

    @property
    def is_new(self):
        return self.attendee.data == 'other'

    @property
    def css_class(self):
        return len(self.attendee.choices) == 1 and 'hide-attendee'

    def populate_attendees(self):
        assert self.request.is_logged_in

        attendees = AttendeeCollection(self.request.app.session())
        attendees = attendees.by_username(self.request.current_username)
        attendees = attendees.with_entities(Attendee.id, Attendee.name)
        attendees = attendees.order_by(Attendee.name)

        self.attendee.choices = [(a.id.hex, a.name) for a in attendees.all()]
        self.attendee.choices.append(('other', _("Other")))

        # override the default
        if self.attendee.data == '0xdeadbeef':
            self.attendee.data = self.attendee.choices[0][0]

    def on_request(self):
        self.populate_attendees()

    def ensure_no_duplicate_child(self):
        if self.is_new:
            attendees = AttendeeCollection(self.request.app.session())
            query = attendees.by_username(self.request.current_username)
            query = query.filter(Attendee.name == self.name.data.strip())

            if query.first():
                self.name.errors.append(
                    _("You already entered a child with this name"))

                return False

        return True

    def ensure_no_duplicate_booking(self):
        if not self.is_new:
            bookings = BookingCollection(self.request.app.session())
            query = bookings.by_occasion(self.model)
            query = query.filter(Booking.attendee_id == self.attendee.data)

            booking = query.first()

            if booking:
                self.attendee.errors.append(_(
                    "This occasion has already been booked by ${name}",
                    mapping={'name': booking.attendee.name}
                ))

                return False

        return True

    def ensure_active_period(self):
        if not self.model.period.active:
            self.attendee.errors.append(_(
                "This occasion is outside the currently active period"
            ))
            return False

        return True

    def ensure_within_prebooking_period(self):
        start, end = (
            self.model.period.prebooking_start,
            self.model.period.prebooking_end
        )
        today = date.today()

        if not (start <= today and today <= end):
            self.attendee.errors.append(_(
                "Cannot create a booking outside the prebooking period"
            ))
            return False

        return True

    def validate(self):
        result = super().validate()

        ensurances = (
            self.ensure_active_period,
            self.ensure_within_prebooking_period,
            self.ensure_no_duplicate_child,
            self.ensure_no_duplicate_booking
        )

        for ensurance in ensurances:
            if not ensurance():
                result = False

        return result

==> onegov.feriennet-0.1.1.tar.gz: onegov.feriennet-0.1.1/onegov/feriennet/forms/attendee.py <==
from cached_property import cached_property
from datetime import date
from onegov.activity import Attendee, AttendeeCollection
from onegov.activity import Booking, BookingCollection
from onegov.feriennet import _
from onegov.form import Form
from onegov.user import UserCollection
from purl import URL
from wtforms.fields import RadioField, StringField
from wtforms.fields.html5 import DateField
from wtforms.validators import InputRequired


class AttendeeForm(Form):

    attendee = RadioField(
        label=_("Attendee"),
        validators=[InputRequired()],
        default='0xdeadbeef')

    name = StringField(
        label=_("Full Name"),
        validators=[InputRequired()],
        depends_on=('attendee', 'other'))

    birth_date = DateField(
        label=_("Birthdate"),
        validators=[InputRequired()],
        depends_on=('attendee', 'other'))

    @property
    def is_new(self):
        return self.attendee.data == 'other'

    @property
    def css_class(self):
        return len(self.attendee.choices) == 1 and 'hide-attendee'

    @cached_property
    def username(self):
        if not self.request.is_admin:
            return self.request.current_username

        return self.request.params.get(
            'username', self.request.current_username)

    @cached_property
    def user(self):
        users = UserCollection(self.request.app.session())
        return users.by_username(self.username)

    def for_username(self, username):
        url = URL(self.action)
        url = url.query_param('username', username)

        return url.as_string()

    def populate_attendees(self):
        assert self.request.is_logged_in

        attendees = AttendeeCollection(self.request.app.session())
        attendees = attendees.by_username(self.username)
        attendees = attendees.with_entities(Attendee.id, Attendee.name)
        attendees = attendees.order_by(Attendee.name)

        self.attendee.choices = [(a.id.hex, a.name) for a in attendees.all()]
        self.attendee.choices.append(('other', _("Other")))

        # override the default
        if self.attendee.data == '0xdeadbeef':
            self.attendee.data = self.attendee.choices[0][0]

    def on_request(self):
        self.populate_attendees()

    def ensure_no_duplicate_child(self):
        if self.is_new:
            attendees = AttendeeCollection(self.request.app.session())
            query = attendees.by_username(self.username)
            query = query.filter(Attendee.name == self.name.data.strip())

            if query.first():
                self.name.errors.append(
                    _("You already entered a child with this name"))

                return False

        return True

    def ensure_no_duplicate_booking(self):
        if not self.is_new:
            bookings = BookingCollection(self.request.app.session())
            query = bookings.by_occasion(self.model)
            query = query.filter(Booking.attendee_id == self.attendee.data)

            booking = query.first()

            if booking:
                self.attendee.errors.append(_(
                    "This occasion has already been booked by ${name}",
                    mapping={'name': booking.attendee.name}
                ))

                return False

        return True

    def ensure_active_period(self):
        if not self.model.period.active:
            self.attendee.errors.append(_(
                "This occasion is outside the currently active period"
            ))
            return False

        return True

    def ensure_within_prebooking_period(self):
        start, end = (
            self.model.period.prebooking_start,
            self.model.period.prebooking_end
        )
        today = date.today()

        if not (start <= today and today <= end):
            self.attendee.errors.append(_(
                "Cannot create a booking outside the prebooking period"
            ))
            return False

        return True

    def ensure_available_spots(self):
        if self.model.full and not self.model.period.wishlist_phase:
            self.attendee.errors.append(_(
                "This occasion is already fully booked"
            ))
            return False

        return True

    def ensure_no_conflict(self):
        if self.model.period.confirmed:
            bookings = BookingCollection(
                session=self.request.app.session(),
                period_id=self.model.period_id)

            query = bookings.query()
            query = query.filter(Booking.attendee_id == self.attendee.data)
            query = query.filter(Booking.state == 'accepted')

            for booking in query:
                if booking.overlaps(self.model):
                    self.attendee.errors.append(_(
                        "This occasion overlaps with another booking"
                    ))
                    return False

        return True

    def validate(self):
        result = super().validate()

        ensurances = (
            self.ensure_active_period,
            self.ensure_within_prebooking_period,
            self.ensure_no_duplicate_child,
            self.ensure_no_duplicate_booking,
            self.ensure_available_spots,
            self.ensure_no_conflict,
        )

        for ensurance in ensurances:
            if not ensurance():
                result = False

        return result
